/* cmd_neonserv_access.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonserv.h"

/*
* argv[0] - nick / *auth
*/
static USERAUTH_CALLBACK(neonserv_cmd_access_nick_lookup);
static void neonserv_cmd_access_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, char *nick, char *auth, struct UserNode *target);
static void neonserv_cmd_access_filter_ranking_info(char *info, char *buffer, char *nick, char *auth, char *rank_name);

struct neonserv_cmd_access_cache {
    struct ClientSocket *client, *textclient;
    struct UserNode *user;
    struct ChanNode *chan;
    char *nick;
};

CMD_BIND(neonserv_cmd_access) {
    if(argc == 0) {
        if(!(user->flags & USERFLAG_ISAUTHED)) {
            struct neonserv_cmd_access_cache *cache = malloc(sizeof(*cache));
            if (!cache) {
                perror("malloc() failed");
                return;
            }
            cache->client = client;
            cache->textclient = textclient;
            cache->user = user;
            cache->chan = chan;
            cache->nick = strdup(user->nick);
            get_userauth(user, module_id, neonserv_cmd_access_nick_lookup, cache);
        } else
            neonserv_cmd_access_async1(client, textclient, user, chan, user->nick, user->auth, user);
    }
    else if(argv[0][0] == '*') {
        //we've got an auth
        argv[0]++;
        neonserv_cmd_access_async1(client, textclient, user, chan, NULL, argv[0], NULL);
    } else {
        struct UserNode *cuser = getUserByNick(argv[0]);
        if(!cuser) {
            cuser = createTempUser(argv[0]);
            if(!cuser) {
                reply(textclient, user, "NS_USER_UNKNOWN", argv[0]);
                return;
            }
            cuser->flags |= USERFLAG_ISTMPUSER;
        }
        if(cuser->flags & USERFLAG_ISAUTHED) {
            neonserv_cmd_access_async1(client, textclient, user, chan, argv[0], cuser->auth, cuser);
        } else {
            struct neonserv_cmd_access_cache *cache = malloc(sizeof(*cache));
            if (!cache) {
                perror("malloc() failed");
                return;
            }
            cache->client = client;
            cache->textclient = textclient;
            cache->user = user;
            cache->chan = chan;
            cache->nick = strdup(argv[0]);
            get_userauth(cuser, module_id, neonserv_cmd_access_nick_lookup, cache);
        }
    }
}

static USERAUTH_CALLBACK(neonserv_cmd_access_nick_lookup) {
    struct neonserv_cmd_access_cache *cache = data;
    if(!user) {
        //USER_DOES_NOT_EXIST
        reply(cache->textclient, cache->user, "NS_USER_UNKNOWN", cache->nick);
    }
    else if(!(user->flags & USERFLAG_ISAUTHED)) {
        //USER_NOT_AUTHED
        if(!strcmp(cache->nick, cache->user->nick))
            reply(cache->textclient, cache->user, "NS_YOU_NEED_AUTH");
        else
            reply(cache->textclient, cache->user, "NS_USER_NEED_AUTH", cache->nick);
    }
    else
        neonserv_cmd_access_async1(cache->client, cache->textclient, cache->user, cache->chan, user->nick, user->auth, user);
    free(cache->nick);
    free(cache);
}

static void neonserv_cmd_access_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan, char *nick, char *auth, struct UserNode *target) {
    //we've got a valid auth now...
    MYSQL_RES *res;
    MYSQL_ROW user_row, chanuser_row;
    int userid;
    printf_mysql_query("SELECT `user_id`, `user_access`, `user_god`, `user_rank`, `rank_info`, `rank_name` FROM `users` LEFT JOIN `support_ranks` ON `rank_id` = `user_rank` WHERE `user_user` = '%s'", escape_string(auth));
    res = mysql_use();
    if ((user_row = mysql_fetch_row(res)) != NULL) {
        userid = atoi(user_row[0]);
        //check if the user is already added
        printf_mysql_query("SELECT `chanuser_access`, `chanuser_flags`, `chanuser_infoline` FROM `chanusers` WHERE `chanuser_cid` = '%d' AND `chanuser_uid` = '%d'", chan->channel_id, userid);
        res = mysql_use();
        if(strcmp(user_row[3], "0") && strcmp(user_row[4], "")) {
            char rank_info[MAXLEN];
            neonserv_cmd_access_filter_ranking_info(user_row[4], rank_info, nick, auth, user_row[5]);
            reply(textclient, user, "%s", rank_info);
        }
        if ((chanuser_row = mysql_fetch_row(res)) != NULL) {
            //access output
            if(nick)
                reply(textclient, user, (strcmp(user_row[2], "1") ? "NS_A_ACCESS_AUTH" : "NS_A_ACCESS_AUTH_GOD"), nick, auth, atoi(chanuser_row[0]), chan->name);
            else
                reply(textclient, user, (strcmp(user_row[2], "1") ? "NS_A_ACCESS_NICK" : "NS_A_ACCESS_NICK_GOD"), auth, atoi(chanuser_row[0]), chan->name);
            int cflags = atoi(chanuser_row[1]);
            if(cflags & DB_CHANUSER_SUSPENDED)
                reply(textclient, user, "NS_A_SUSPENDED", (nick ? nick : auth), chan->name);
            if(chanuser_row[2] && strcmp(chanuser_row[2], ""))
                reply(textclient, user, "[%s] %s", (nick ? nick : auth), chanuser_row[2]);
        } else if(!strcmp(user_row[2], "1")) {
            if(nick)
                reply(textclient, user, "NS_A_LACKS_ACCESS_BUT_GOD_AUTH", nick, auth, chan->name);
            else
                reply(textclient, user, "NS_A_LACKS_ACCESS_BUT_GOD_NICK", auth, chan->name);
        } else
            reply(textclient, user, "NS_NOT_ON_USERLIST", (nick ? nick : auth), chan->name);
    } else
        reply(textclient, user, "NS_NOT_ON_USERLIST", (nick ? nick : auth), chan->name);
    if(target && (target->flags & USERFLAG_ISIRCOP))
        reply(textclient, user, "NS_A_IS_IRCOP", nick);
}

static void neonserv_cmd_access_filter_ranking_info(char *info, char *buffer, char *nick, char *auth, char *rank_name) {
    int bufferPos = 0;
    char *a, *b = info;
    do {
        if(!b) break;
        a = strstr(b, "$");
        if(a) *a = '\0';
        bufferPos += sprintf(buffer + bufferPos, "%s", b);
        if(!a) break;
        switch(a[1]) {
            case '\0':
                a = NULL;
                break;
            case 'U':
                bufferPos += sprintf(buffer + bufferPos, "%s", auth);
                break;
            case 'N':
                bufferPos += sprintf(buffer + bufferPos, "%s", (nick ? nick : auth));
                break;
            case 'R':
                bufferPos += sprintf(buffer + bufferPos, "%s", rank_name);
                break;
            default:
                buffer[bufferPos++] = '$';
                buffer[bufferPos++] = a[1];
                break;
        }
        if(a)
            b = a+2;
    } while(a);
}
