/* cmd_neonserv_assignrank.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonserv.h"

/*
* argv[0] - nick / *auth
* argv[1-*] - rank name
*/
static AUTHLOOKUP_CALLBACK(neonserv_cmd_assignrank_auth_lookup);
static USERAUTH_CALLBACK(neonserv_cmd_assignrank_nick_lookup);
static void neonserv_cmd_assignrank_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct Event *event, char *nick, char *auth, int rank_id);

struct neonserv_cmd_assignrank_cache {
    struct ClientSocket *client, *textclient;
    struct UserNode *user;
    struct Event *event;
    int rank_id;
    char *nick;
};

CMD_BIND(neonserv_cmd_assignrank) {
    MYSQL_RES *res;
    MYSQL_ROW row;
    char *name = merge_argv(argv, 1, argc);
    int rank_id = 0;
    if(stricmp(name, "*") && stricmp(name, "user") && stricmp(name, "none")) {
        printf_mysql_query("SELECT `rank_id`, `rank_name` FROM `support_ranks` WHERE `rank_name` = '%s'", escape_string(name));
        res = mysql_use();
        if ((row = mysql_fetch_row(res)) == NULL) {
            reply(textclient, user, "NS_DELRANK_NOT_FOUND", name);
            return;
        }
        rank_id = atoi(row[0]);
    }
    if(argv[0][0] == '*') {
        //we've got an auth
        argv[0]++;
        printf_mysql_query("SELECT `user_user` FROM `users` WHERE `user_user` = '%s'", escape_string(argv[0]));
        res = mysql_use();
        if ((row = mysql_fetch_row(res)) != NULL) {
            neonserv_cmd_assignrank_async1(client, textclient, user, event, argv[0], row[0], rank_id);
        } else {
            //we need to create a new user...
            //but first lookup the auth to check if it really exists
            struct neonserv_cmd_assignrank_cache *cache = malloc(sizeof(*cache));
            if (!cache) {
                perror("malloc() failed");
                return;
            }
            cache->client = client;
            cache->textclient = textclient;
            cache->user = user;
            cache->event = event;
            cache->rank_id = rank_id;
            cache->nick = strdup(argv[0]);
            lookup_authname(argv[0], module_id, neonserv_cmd_assignrank_auth_lookup, cache);
        }
    } else {
        struct UserNode *cuser = getUserByNick(argv[0]);
        if(!cuser) {
            cuser = createTempUser(argv[0]);
			if(!cuser) {
                reply(textclient, user, "NS_USER_UNKNOWN", argv[0]);
                return;
            }
            cuser->flags |= USERFLAG_ISTMPUSER;
        }
        if(cuser->flags & USERFLAG_ISAUTHED) {
            neonserv_cmd_assignrank_async1(client, textclient, user, event, argv[0], cuser->auth, rank_id);
        } else {
            struct neonserv_cmd_assignrank_cache *cache = malloc(sizeof(*cache));
            if (!cache) {
                perror("malloc() failed");
                return;
            }
            cache->client = client;
            cache->textclient = textclient;
            cache->user = user;
            cache->event = event;
            cache->rank_id = rank_id;
            cache->nick = strdup(argv[0]);
            get_userauth(cuser, module_id, neonserv_cmd_assignrank_nick_lookup, cache);
        }
    }
}

static AUTHLOOKUP_CALLBACK(neonserv_cmd_assignrank_auth_lookup) {
    struct neonserv_cmd_assignrank_cache *cache = data;
    if(!exists) {
        //AUTH_DOES_NOT_EXIST
        reply(cache->textclient, cache->user, "NS_AUTH_UNKNOWN", cache->nick);
    } else
        neonserv_cmd_assignrank_async1(cache->client, cache->textclient, cache->user, cache->event, cache->nick, auth, cache->rank_id);
    free(cache->nick);
    free(cache);
}

static USERAUTH_CALLBACK(neonserv_cmd_assignrank_nick_lookup) {
    struct neonserv_cmd_assignrank_cache *cache = data;
    if(!user) {
        //USER_DOES_NOT_EXIST
        reply(cache->textclient, cache->user, "NS_USER_UNKNOWN", cache->nick);
    }
    else if(!(user->flags & USERFLAG_ISAUTHED)) {
        //USER_NOT_AUTHED
        reply(cache->textclient, cache->user, "NS_USER_NEED_AUTH", cache->nick);
    }
    else
        neonserv_cmd_assignrank_async1(cache->client, cache->textclient, cache->user, cache->event, user->nick, user->auth, cache->rank_id);
    free(cache->nick);
    free(cache);
}

static void neonserv_cmd_assignrank_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct Event *event, char *nick, char *auth, int rank_id) {
    //we've got a valid auth now...
    MYSQL_RES *res;
    MYSQL_ROW row, rank = NULL;
    int caccess = 0;
    if(rank_id) {
        printf_mysql_query("SELECT `rank_access`, `rank_name` FROM `support_ranks` WHERE `rank_id` = '%d'", rank_id);
        res = mysql_use();
        rank = mysql_fetch_row(res);
        caccess = atoi(rank[0]);
    }
    printf_mysql_query("SELECT `user_id`, `user_rank` FROM `users` WHERE `user_user` = '%s'", escape_string(auth));
    res = mysql_use();
    if ((row = mysql_fetch_row(res)) != NULL) {
        if(atoi(row[1]) != rank_id) {
            printf_mysql_query("UPDATE `users` SET `user_access` = '%d', `user_rank` = '%d' WHERE `user_id` = '%s'", caccess, rank_id, row[0]);
        }
    } else if(rank_id) {
        printf_mysql_query("INSERT INTO `users` (`user_user`, `user_access`, `user_rank`) VALUES ('%s', '%d', '%d')", escape_string(auth), caccess, rank_id);
    }
    reply(textclient, user, "NS_ASSIGNRANK_DONE", auth, (rank_id ? rank[1] : "*"));
    logEvent(event);
}
